unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, KbdTable;

type
  TMainForm = class(TForm)
    btnParseLogFile: TButton;
    btnLoadKbdLib: TButton;
    KbdLibName: TEdit;
    lbl1: TLabel;
    grpMain: TGroupBox;
    dlgOpenLogFile: TOpenDialog;
    edtFileName: TEdit;
    lbl2: TLabel;
    btnBrowse: TButton;
    dlgSaveParsedData: TSaveDialog;
    lbl3: TLabel;
    procedure btnLoadKbdLibClick(Sender: TObject);
    procedure btnParseLogFileClick(Sender: TObject);
    procedure btnBrowseClick(Sender: TObject);
  private
    procedure ScanCodesToKeyNames(Buffer: PWord; BufferSize: DWORD; FileName: PChar);
    { Private declarations }
  public
    CurrentLib: HMODULE;
    CurrentKBDTable: PKBDTABLES;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.btnLoadKbdLibClick(Sender: TObject);
var
  GetTableFunc:TGetKbdLayerDescriptorFunc;
begin
  CurrentLib := LoadLibrary(PChar(KbdLibName.Text));
  if CurrentLib=0 then
   begin
    ShowMessage('  !');
    exit;
   end;

  GetTableFunc:= TGetKbdLayerDescriptorFunc(GetProcAddress(CurrentLib, 'KbdLayerDescriptor'));
  CurrentKBDTable := GetTableFunc();
  grpMain.Enabled:=True;
end;

var
  KeyPressed: array[0..6] of WCHAR = ('D', 'O', 'W', 'N', ' ', ':', ' ');
  KeyReleased: array[0..4] of WCHAR = ('U', 'P', ' ', ':', ' ');
  NewLine: array[0..1] of WCHAR = (#13, #10);

procedure TMainForm.ScanCodesToKeyNames(Buffer: PWord; BufferSize:DWORD; FileName:PChar);
var
  KeyName : array[0..23] of WCHAR;
  CurrKeyName: PWChar;
  FileHandle:THandle;
  _Writed:DWORD;
  _i, _c:Integer;
  cVK:Word;
begin
  FileHandle:=CreateFile(FileName, GENERIC_WRITE, FILE_SHARE_READ, 0, CREATE_ALWAYS, 0, 0);
  if FileHandle=INVALID_HANDLE_VALUE then Exit;
  _c:=BufferSize div sizeof(Word);
  for _i:=1 to _c do
   begin
    if ((Buffer^ shr 8) and KEY_BREAK)=KEY_BREAK then
     WriteFile(FileHandle, KeyReleased, SizeOf(KeyReleased), _Writed, 0)
                                               else
     WriteFile(FileHandle, KeyPressed, SizeOf(KeyPressed), _Writed, 0);

    CurrKeyName:=GetKeyNameW(CurrentKBDTable, (Buffer^ and $FF));
    if CurrKeyName<>nil then
      WriteFile(FileHandle, CurrKeyName^, lstrlenW(CurrKeyName)*2, _Writed, 0)
                        else
      begin
        cVK:=GetVirtualCodeFromScanCode(CurrentKBDTable, Buffer^ and $FF);
        WriteFile(FileHandle, cVK, SizeOf(cVK), _Writed, 0);
      end;
    WriteFile(FileHandle, NewLine, 4, _Writed, 0);
    Inc(Buffer);
   end;
  CloseHandle(FileHandle); 
end;

procedure TMainForm.btnParseLogFileClick(Sender: TObject);
var
  FileHandle:THandle;
  FileSize, _Readed: DWORD;
  Buffer: PWord;
begin
  FileHandle:=CreateFile(PChar(edtFileName.Text), GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0);
  if FileHandle=INVALID_HANDLE_VALUE then Exit;
  if dlgSaveParsedData.Execute then
   begin
    FileSize:=GetFileSize(FileHandle, nil);
    Buffer:=VirtualAlloc(0, FileSize, MEM_COMMIT+MEM_RESERVE, PAGE_READWRITE);
    if Buffer <> nil then
     begin
      ReadFile(FileHandle, Buffer^, FileSize, _Readed, 0);

      ScanCodesToKeyNames(Buffer, FileSize, PChar(dlgSaveParsedData.FileName));
      VirtualFree(Buffer, 0, MEM_RELEASE);
     end;
   end;
  CloseHandle(FileHandle);
  //MessageBoxW(0,GetKeyNameW(CurrentKBDTable, $5D),'test',0)
end;

procedure TMainForm.btnBrowseClick(Sender: TObject);
begin
  if dlgOpenLogFile.Execute then
   begin
    edtFileName.Text:=dlgOpenLogFile.FileName;
   end;
end;

end.
  